/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String
import moka.lang.Character;//Import the class moka.lang.Character
import moka.lang.Integer;//Import the class moka.lang.Integer

/**
 * Some static functions used by the other classes.
 *
 * @author  Frdric Brown
 */
public class DisASM {
	public static char ** opVecStr;
	public static Format[] opVecForm;

	public static final short TAILLE_TABLEAUX = 256;
	public static final short INDEX_MAX = 255;

	static {
		DisASM.opVecStr = native.malloc(256 * sizeof(char*));
		native.memset(DisASM_opVecStr, 0, 256);
		DisASM.opVecForm = new Format[256];
		native.memset(DisASM_opVecForm, 0, 256);
	}

	finally {
		native.free(DisASM_opVecStr);
		native.free(DisASM_opVecForm);
	}

	public static short trouverIndex(char[] str) {
		for (short i=0; i < DisASM.TAILLE_TABLEAUX; i++) {
			if (!native.strcmp(DisASM_opVecStr[i], str)) {
				return i;
			}
		}

		return -1;
	}

    /**
     * The main function is the entry point of a Moka program.
    */
    public static void main () {
		String expr = System.getString();

		char * exprP = expr.value;
		native {
			char * endP = &exprP[strlen(exprP)];
		}
		BYTE opCode;

		native {
			clrscr();
			printf("Expression : ");
			printf(exprP);
			printf("\n");

		}

		while (exprP < endP) {

			opCode = DisASM.getByte(exprP, 8);
			exprP += 2;

			if (opCode < 0x40) {
				if ((exprP + 2) > endP) {
					break;
				}
			}
			else if (opCode < 0xC0) {
				if ((exprP + 6) > endP) {
					break;
				}
			}
			else {
				if ((exprP + 10) > endP) {
					break;
				}
			}

			if (DisASM.opVecStr[opCode]) {
				System.println(DisASM.opVecForm[opCode].print((BYTE)opCode, exprP));
				System.read();
			}
			else {

				System.print("OpCode inconnu : 0x");

				System.println(DisASM.printHex8((BYTE)opCode));

				System.read();
			}

			if (opCode < 0x40) {
				exprP += 2;
			}
			else if (opCode < 0xC0) {
				exprP += 6;
			}
			else {
				exprP += 10;
			}
		}

		System.print("Programme dassembl!");

		System.read();

		expr.finalize();
	}

	public native static String printHex12(short s) {
		char buf[4] = "   ";
		short i = 3;


		while (--i >= 0) {
			if ((s & 0x0F) <= 9) {
				buf[i] = (short) (s & 0x0F) + '0';
			}
			else {
				buf[i] = (short) (s & 0x0F) - 10 + 'A';
			}

			s = s >> 4;
		}

		return TString_char_p_BOOL(Character_copyValueOf_char_p(buf), FALSE);
	}

	public static int numValue(char[] str) {
		int value;
		char bufH[3];
		char bufB[9];
		short i;

		if (str[0] == 'B') {
			str ++;
			if (str[0]) str ++;

			for (i = 0; (str[0] && (str[0] != '\'' && str[0] != '\"')) && (i < 8); i++) {
				bufB[i] = str[i];
			}

			bufB[i] = '\0';

			value = DisASM.getBin((char[])bufB);
		}
		else if (str[0] == 'X') {
			str ++;
			if (str[0]) str ++;

			for (i = 0; (str[0] && (str[0] != '\'' && str[0] != '\"')) && (i < 2); i++) {
				bufH[i] = str[i];
			}

			bufH[i] = '\0';

			value = DisASM.getByte((char[])bufH, 8);
		}
		else {
			value = Integer.parseInt(new String(str));
		}

		return value;
	}

	public static ubyte fetchRegister(void ** strP) {
		char* str = *strP;
		char buf[25];
		short i;

		while (str[0] == ' ') str++;

		for (i=0; str[0] && str[0] != ' ' && str[0] != '\n' && str[0] != ',' && str[0] != ')'; i++) {
			buf[i] = *str++;
		}

		buf[i] = '\0';

		while (str[0] == ' ') str++;

		if (str[0] == ')') {
			str++;
		}

		while (str[0] == ' ') str++;

		if (str[0] == ',' || str[0] == '(') {
			str++;
		}

		*strP = str;

		return DisASM.numValue(buf);
	}

	public static ubyte fetchValue(void ** strP) {
		char* str = *strP;
		char buf[25];
		short i;

		//native.printf("%c", str[0]); native.ngetchx();

		while (str[0] == ' ') str++;

		if (str[0] == ',') {
			str++;

			while (str[0] == ' ') str++;
		}

		for (i=0; str[0] && str[0] != ' ' && str[0] != '\n' && str[0] != ',' && str[0] != ')'; i++) {
			buf[i] = *str++;
		}

		buf[i] = '\0';

		while (str[0] == ' ') str++;

		if (str[0] == ',' || str[0] == '(') {
			str++;
		}

		*strP = str;

		return DisASM.numValue(buf);
	}

	public static ushort fetchD(void ** strP) {
		char* str = *strP;
		char buf[25];
		short i;

		while (str[0] == ' ' || str[0] == ',' || str[0] == '(' || str[0] == ')') str++;

		for (i=0; str[0] && str[0] != ' ' && str[0] != '\n' && str[0] != ',' && str[0] != '(' && str[0] != ')'; i++) {
			buf[i] = *str++;
		}

		buf[i] = '\0';

		while (str[0] == ' ') str++;

		if (str[0] == ',' || str[0] == '(' || str[0] == ')') {
			str++;
		}

		*strP = str;

		return DisASM.numValue(buf);
	}

	public static ubyte fetchL(void ** strP) {
		char* str = *strP;
		char buf[25];
		short i;

		while (str[0] == ' ' || str[0] == '(' || str[0] == ')') str++;

		if (str[0] == ',') {
			str ++;

			*strP = str;

			return 1;
		}

		for (i=0; str[0] && str[0] != ' ' && str[0] != '\n' && str[0] != ',' && str[0] != '(' && str[0] != ')'; i++) {
			buf[i] = *str++;
		}

		buf[i] = '\0';

		while (str[0] == ' ') str++;

		if (str[0] == ',' || str[0] == '(' || str[0] == ')') {
			str++;
		}

		*strP = str;

		return DisASM.numValue(buf);
	}

	public native static String printHex8(ubyte b, short i) {
		char buf[3] = "  ";

		buf[i] = '\0';

		while (--i >= 0) {
			if ((b & 0x0F) <= 9) {
				buf[i] = (char) (b & 0x0F) + '0';
			}
			else {
				buf[i] = (char) (b & 0x0F) - 10 + 'A';
			}

			b = b >> 4;
		}

		return TString_char_p_BOOL(Character_copyValueOf_char_p(buf), FALSE);
	}

	public native static String printHex8(ubyte b) {
		char buf[3] = "  ";
		short i = 2;


		while (--i >= 0) {
			if ((b & 0x0F) <= 9) {
				buf[i] = (char) (b & 0x0F) + '0';
			}
			else {
				buf[i] = (char) (b & 0x0F) - 10 + 'A';
			}

			b = b >> 4;
		}

		return TString_char_p_BOOL(Character_copyValueOf_char_p(buf), FALSE);
	}

	public native static String printBin4(ubyte b) {
		char buf[5] = "    ";
		short i = 4;

		while (--i >= 0) {
			if (b & 0x01) {
				buf[i] = '1';
			}
			else {
				buf[i] = '0';
			}

			b = b >> 1;
		}

		return TString_char_p_BOOL(Character_copyValueOf_char_p(buf), FALSE);
	}

	public static ubyte getByte(char * str, short shift) {
		ubyte value = 0;

		for (short i = 0; (shift-=4) >= 0; i ++) {
			if (str[i] <= '9') {
				value += (BYTE)(str[i] - '0') << shift;
			}
			else {
				value += (BYTE)(str[i] - 'A' + 10) << shift;
			}
		}

		return value;
	}

	public static ubyte getBin(char * str) {
		ubyte value = 0;
		short shift = 8;

		for (short i = 0; --shift >= 0; i ++) {
			value += (BYTE)(str[i] - '0') << shift;
		}

		return value;
	}

	public static ushort getHalf(char * str, short shift) {
		ushort value = 0;

		for (short i = 0; (shift-=4) >= 0; i ++) {
			if (str[i] <= '9') {
				value += ((ushort)str[i] - '0') << shift;

			}
			else {
				value += ((ushort)str[i] - 'A' + 10) << shift;
			}
		}

		return value;
	}

}