/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String

/**
 * Format OpCode R1,R2
 *
 * @author  Frdric Brown
 */
public class FormatRR0 extends Format {

	private static FormatRR0 form;

    public String print (ubyte opCode, char *exprP) {
		NYBBLE R1;
		NYBBLE R2;

		R1 = DisASM.getByte(exprP++, 4);

		R2 = DisASM.getByte(exprP++, 4);

		String str = new String(DisASM.opVecStr[opCode]);

		str += " ";

		str += ((short)native.R1);

		str += ",";

		str += ((short)native.R2);

		return str;
	}

    public String assemble (ubyte opCode, char *exprP) {
		NYBBLE R1 = 0;
		NYBBLE R2 = 0;

		R1 = DisASM.fetchRegister(&exprP);

		R2 = DisASM.fetchRegister(&exprP);

		String str = DisASM.printHex8(opCode);

		str += DisASM.printHex8((ubyte)R1, 1);

		str += DisASM.printHex8((ubyte)R2, 1);

		return str;
	}


	static {
		Format f = FormatRR0.form = new FormatRR0();
		char ** opVecStr = DisASM.opVecStr;
		Format[] opVecForm = DisASM.opVecForm;

		native {
			opVecStr[0x05] = "BALR";
			opVecForm[0x05] = f;

			opVecStr[0x06] = "BCTR";
			opVecForm[0x06] = f;

			opVecStr[0x0D] = "BASR";
			opVecForm[0x0D] = f;

			opVecStr[0x0E] = "MVCL";
			opVecForm[0x0E] = f;

			opVecStr[0x12] = "LTR";
			opVecForm[0x12] = f;

			opVecStr[0x14] = "NR";
			opVecForm[0x14] = f;

			opVecStr[0x15] = "CLR";
			opVecForm[0x15] = f;

			opVecStr[0x16] = "OR";
			opVecForm[0x16] = f;

			opVecStr[0x18] = "LR";
			opVecForm[0x18] = f;

			opVecStr[0x19] = "CR";
			opVecForm[0x19] = f;

			opVecStr[0x1A] = "AR";
			opVecForm[0x1A] = f;

			opVecStr[0x1B] = "SR";
			opVecForm[0x1B] = f;

			opVecStr[0x1C] = "MR";
			opVecForm[0x1C] = f;

			opVecStr[0x1D] = "DR";
			opVecForm[0x1D] = f;
		}
	}

	finally {
		FormatRR0.form.finalize();
	}
}