/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String

/**
 * Format OpCode D1(L1,B1),D2(L2,B2)
 *
 * @author  Frdric Brown
 */
public class FormatSS1 extends Format {

	private static FormatSS1 form;

    public String print (ubyte opCode, char *exprP) {
		BITS12 D1;
		NYBBLE L1;
		NYBBLE B1;
		BITS12 D2;
		NYBBLE L2;
		NYBBLE B2;

		L1 = DisASM.getByte(exprP++, 4);

		L2 = DisASM.getByte(exprP++, 4);

		B1 = DisASM.getByte(exprP++, 4);

		D1 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		B2 = DisASM.getByte(exprP++, 4);

		D2 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		String str = new String(DisASM.opVecStr[opCode]);

		str += " ";

		str += ((short)native.D1);

		str += "(";

		str += ((short)native.++L1);

		str += ",";

		str += ((short)native.B1);

		str += "),";

		str += ((short)native.D2);

		str += "(";

		str += ((short)native.++L2);

		str += ",";

		str += ((short)native.B2);

		str += ")";

		return str;
	}

    public String assemble (ubyte opCode, char *exprP) {
		BITS12 D1;
		NYBBLE L1;
		NYBBLE B1;
		BITS12 D2;
		NYBBLE L2;
		NYBBLE B2;

		D1 = DisASM.fetchD(&exprP);

		L1  = (ubyte)DisASM.fetchL(&exprP);

		B1 = DisASM.fetchRegister(&exprP);

		D2 = DisASM.fetchL(&exprP);

		L2  = (ubyte)DisASM.fetchL(&exprP);

		B2 = DisASM.fetchRegister(&exprP);

		String str = DisASM.printHex8(opCode);

		str += DisASM.printHex8((ubyte)--L1, 1);

		str += DisASM.printHex8((ubyte)--L2, 1);

		str += DisASM.printHex8((ubyte)B1, 1);

		str += DisASM.printHex12((short)D1);

		str += DisASM.printHex8((ubyte)B2, 1);

		str += DisASM.printHex12((short)D2);

		return str;
	}

	static {
		Format f = FormatSS1.form = new FormatSS1();
		char ** opVecStr = DisASM.opVecStr;
		Format[] opVecForm = DisASM.opVecForm;

		native {
			opVecStr[0xF2] = "PACK";
			opVecForm[0xF2] = f;

			opVecStr[0xF3] = "UNPK";
			opVecForm[0xF3] = f;

			opVecStr[0xF8] = "ZAP";
			opVecForm[0xF8] = f;

			opVecStr[0xF9] = "CP";
			opVecForm[0xF9] = f;

			opVecStr[0xFA] = "AP";
			opVecForm[0xFA] = f;

			opVecStr[0xFB] = "SP";
			opVecForm[0xFB] = f;

			opVecStr[0xFC] = "MP";
			opVecForm[0xFC] = f;

			opVecStr[0xFD] = "DP";
			opVecForm[0xFD] = f;
		}
	}

	finally {
		FormatSS1.form.finalize();
	}
}