/*
 * This file is the main file of the C project.
 * It includes the packages.h file, defines the appropriate constants,  
 */
#include "features.h"
/* Used by the Moka macro language files */
#define MOKAMACRO_CALLMAIN xStartFunc = GuiASM_main_
/* Used by garbage collection process.*/
#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection || MOKAUSES_GarbageCollector

/* ADT's constants */
#define ADT_GarbageCollector_INCREMENT 10
#define ADT_GarbageCollector_INITIALCAPACITY 10

/* ADT's structure */
typedef struct {
	long int max;
	long int size;
	void * *array;
} ADT_GarbageCollector;

/* ADT's functions and macros */
void ADT_GarbageCollector_init(ADT_GarbageCollector *) ;
//void ADT_GarbageCollector_dispose(ADT_GarbageCollector *) ;
#define ADT_GarbageCollector_dispose(gc) free(gc->array)
void * ADT_GarbageCollector_add(ADT_GarbageCollector *, void *) ;
void * ADT_GarbageCollector_rem(ADT_GarbageCollector *, void *) ;

#if MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p
	void ADT_GarbageCollector_clearPtrs(ADT_GarbageCollector *) ;
#endif /* MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p */

void ADT_GarbageCollector_clearObjects(ADT_GarbageCollector *) ;

#endif /* MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection || MOKAUSES_GarbageCollector */
/* Definition of some constants used by TIGCC.*/
#if MOKASTATICMETHOD_File_showVarLinkDialog_
#define SET_FILE_IN_USE_BIT 	// With this definition, the program's in use (a.k.a. hidden) bit is set while the program is running.
#endif

#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_ROMCallsOptimization && ! MOKASTATICMETHOD_File_showVarLinkDialog_
#define OPTIMIZE_ROM_CALLS    	// Use ROM Call Optimization
#endif 

#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_GhostSpace
#define EXECUTE_IN_GHOST_SPACE 	// Relocate the program in the ghost space
#endif

#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_ErrorReturn
#define ENABLE_ERROR_RETURN     // Enable Returning Errors to TIOS
#endif

#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_Instanciation
	#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_SaveScreen
		#define SAVE_SCREEN
	#endif
#endif

#define byte char				// Define 'byte' as 'char'
#define USE_TI89              	// Compile for TI-89
#define USE_TI92PLUS          	// Compile for TI-92 Plus
#define USE_V200              	// Compile for V200
#include <tigcclib.h>         	// Include All Header Files

#if MOKASTATICMETHOD_System_getMainArgs_
	struct String * * ArgsArray ;
	#define MOKAMACRO_MAINPARMS ArgsArray = System_getMainArgs_ ( )
#else /* MOKASTATICMETHOD_System_getMainArgs_ */
	#define MOKAMACRO_MAINPARMS /* Nothing */
#endif /* MOKASTATICMETHOD_System_getMainArgs_ */

/* Used for the garbage collection process. */
#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection
	ADT_GarbageCollector Objects_GarbageCollector_instance;
	#define GarbageCollector_add_Object(o) ADT_GarbageCollector_add(&Objects_GarbageCollector_instance, o)
	#define GarbageCollector_rem_Object(o) ADT_GarbageCollector_rem(&Objects_GarbageCollector_instance, o)
	
	#if MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p
		ADT_GarbageCollector Ptrs_GarbageCollector_instance;
		#define GarbageCollector_addPtr_void_p(p) ADT_GarbageCollector_add(&Ptrs_GarbageCollector_instance, p)
		#define GarbageCollector_remPtr_void_p(p) ADT_GarbageCollector_rem(&Ptrs_GarbageCollector_instance, p)
	#endif
#endif
/* Used by the serialization and deserialization processes. */
void * * ProtArray; //An array of the object prototypes
#define PROT_NUM 34//The number of prototypes in the prototypes array
/* These lines are required by the Graphic Environment Manager. */
const unsigned char CURSOR_ARROW[] = {0xFF, 0xFE, 0xFC, 0xF8, 0xFC, 0xEE, 0xC7, 0x83};
const unsigned char CURSOR_MASK_ARROW[] = {0x00, 0x7D, 0x7B, 0x77, 0x6B, 0x55, 0x3A, 0x7C};
const unsigned char ICON_CHECK[] = {0x00, 0x02, 0x06, 0x0C, 0x58, 0x70, 0x20, 0x00};
const unsigned char ICON_DOT[] = {0x38, 0x7C, 0xFE, 0xFE, 0xFE, 0x7C, 0x38, 0x00};
const unsigned char ICON_DNARROW[] = {0x18, 0x18, 0x18, 0x18, 0xFF, 0x7E, 0x3C, 0x18};
const unsigned char ICON_UPARROW[] = {0x18, 0x3C, 0x7E, 0xFF, 0x18, 0x18, 0x18, 0x18};
const unsigned char ICON_RARROW[] = {0x08, 0x0C, 0x0E, 0xFF, 0xFF, 0x0E, 0x0C, 0x08};
const unsigned char ICON_LARROW[] = {0x10, 0x30, 0x70, 0xFF, 0xFF, 0x70, 0x30, 0x10};
const unsigned char ICON_UPLEFTCORNER[] = {0xFF, 0xFF, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0};
const unsigned char ICON_UPRIGHTCORNER[] = {0xFF, 0xFF, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03};
const unsigned char ICON_DNLEFTCORNER[] = {0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xFF, 0xFF};
const unsigned char ICON_DNRIGHTCORNER[] = {0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0xFF, 0xFF};
void  (*xStartFunc) ();
#include "packages.h" //Include the packages.h file
#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_Instanciation
	#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_SaveScreen
	void * systemScrSaveBuffer;     // A buffer used to Save/Restore LCD Contents
	#endif	
#endif
/*
 * The entry point of the program, will ultimatly start the Graphic Environment Manager or call the main method of the main class.
 */
void _main ( void ) { 
#if MOKASTATICATTRIBUTE_Graphics_uScrRect
	Graphics_uScrRect = malloc ( sizeof ( SCR_RECT ) ) ; 
	* Graphics_uScrRect = ( SCR_RECT ) { { 0 , 0 , 239 , 127 } } ; 
#endif

#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_SaveScreen
	systemScrSaveBuffer = malloc ( LCD_SIZE ) ; 
	LCD_save ( systemScrSaveBuffer ) ; 
#endif

#if MOKASTATICATTRIBUTE_GEM_vScrRect
#if MOKASTATICATTRIBUTE_Graphics_uScrRect
	GEM_vScrRect = Graphics_uScrRect ; 
#else /* MOKASTATICATTRIBUTE_Graphics_uScrRect */
	GEM_vScrRect = malloc ( sizeof ( SCR_RECT ) ) ; 
	* GEM_vScrRect = ( SCR_RECT ) { { 0 , 0 , 239 , 127 } } ; 
#endif /* MOKASTATICATTRIBUTE_Graphics_uScrRect */
#endif /* MOKASTATICATTRIBUTE_GEM_vScrRect */

#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection
	ADT_GarbageCollector_init(& Objects_GarbageCollector_instance);
	
	#if MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p
		ADT_GarbageCollector_init(& Ptrs_GarbageCollector_instance);
	#endif /* MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p */
#endif /* MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection */

#if MOKASTATICATTRIBUTE_System_CALCULATOR
	System_CALCULATOR = CALCULATOR ; 
#endif /* MOKASTATICATTRIBUTE_System_CALCULATOR */
	ProtArray = malloc ( sizeof ( TObject * ) * 34 ) ; 
	ProtArray [ 0 ] = GuiASM = TGuiASM0 ( ) ; 
	DisASM_static_ ( ) ; 
	ProtArray [ 1 ] = Format = TFormat0 ( ) ; 
	FormatRR0_static_ ( ) ; 
	ProtArray [ 2 ] = FormatRR0 = TFormatRR00 ( ) ; 
	FormatRR1_static_ ( ) ; 
	ProtArray [ 3 ] = FormatRR1 = TFormatRR10 ( ) ; 
	FormatRS0_static_ ( ) ; 
	ProtArray [ 4 ] = FormatRS0 = TFormatRS00 ( ) ; 
	FormatRS1_static_ ( ) ; 
	ProtArray [ 5 ] = FormatRS1 = TFormatRS10 ( ) ; 
	FormatRS2_static_ ( ) ; 
	ProtArray [ 6 ] = FormatRS2 = TFormatRS20 ( ) ; 
	FormatRX0_static_ ( ) ; 
	ProtArray [ 7 ] = FormatRX0 = TFormatRX00 ( ) ; 
	FormatSI0_static_ ( ) ; 
	ProtArray [ 8 ] = FormatSI0 = TFormatSI00 ( ) ; 
	FormatSS0_static_ ( ) ; 
	ProtArray [ 9 ] = FormatSS0 = TFormatSS00 ( ) ; 
	FormatSS1_static_ ( ) ; 
	ProtArray [ 10 ] = FormatSS1 = TFormatSS10 ( ) ; 
	ProtArray [ 11 ] = FrameAbout = TFrameAbout0 ( ) ; 
	ProtArray [ 12 ] = FrameASM = TFrameASM0 ( ) ; 
	System_KEY_DIAMOND = KEY_DIAMOND  ; 
	System_KEY_DOWN = KEY_DOWN  ; 
	System_KEY_LEFT = KEY_LEFT  ; 
	System_KEY_RIGHT = KEY_RIGHT  ; 
	System_KEY_UP = KEY_UP  ; 
	ProtArray [ 13 ] = String = TString0 ( ) ; 
	ProtArray [ 14 ] = Frame = TFrame0 ( ) ; 
	GEM_LCD_HEIGHT = ( System_CALCULATOR ? 128 : 100 )  ; 
	GEM_LCD_WIDTH = ( System_CALCULATOR ? 240 : 160 )  ; 
	GEM_USABLE_HEIGHT = GEM_LCD_HEIGHT - 6  ; 
	ProtArray [ 15 ] = GEM = TGEM_ ( ) ; 
	ProtArray [ 16 ] = Button = TButton0 ( ) ; 
	ProtArray [ 17 ] = TextArea = TTextArea0 ( ) ; 
	ProtArray [ 18 ] = File = TFile0 ( ) ; 
	OptionPane_dialog = NULL  ; 
	ProtArray [ 19 ] = OptionPane = TOptionPane0 ( ) ; 
	ProtArray [ 20 ] = Object = TObject0 ( ) ; 
	ProtArray [ 21 ] = Container = TContainer0 ( ) ; 
	ProtArray [ 22 ] = MenuBar = TMenuBar0 ( ) ; 
	ProtArray [ 23 ] = Component = TComponent0 ( ) ; 
	ProtArray [ 24 ] = EventListener = TEventListener0 ( ) ; 
	Interrupt_busy = FALSE  ; 
	Interrupt_oldInt5Handler = GetIntVec ( AUTO_INT_5 )  ; 
	Interrupt_static_ ( ) ; 
	ProtArray [ 25 ] = Interrupt = TInterrupt0 ( ) ; 
	Timer_static_ ( ) ; 
	RowRead_RR_R_2ND = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 0 ) ) : ~ ( ( short int ) ( 1 << 0 ) ) )  ; 
	RowRead_RR_C_2ND = ( System_CALCULATOR ? ( 1 << 0 ) : ( 1 << 4 ) )  ; 
	RowRead_RR_R_APPS = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 7 ) ) : ~ ( ( short int ) ( 1 << 5 ) ) )  ; 
	RowRead_RR_C_APPS = ( System_CALCULATOR ? ( 1 << 6 ) : ( 1 << 0 ) )  ; 
	RowRead_RR_R_RIGHT = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 0 ) ) : ~ ( ( short int ) ( 1 << 0 ) ) )  ; 
	RowRead_RR_C_RIGHT = ( System_CALCULATOR ? ( 1 << 6 ) : ( 1 << 3 ) )  ; 
	RowRead_RR_R_DOWN = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 0 ) ) : ~ ( ( short int ) ( 1 << 0 ) ) )  ; 
	RowRead_RR_C_DOWN = ( System_CALCULATOR ? ( 1 << 7 ) : ( 1 << 2 ) )  ; 
	RowRead_RR_R_LEFT = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 0 ) ) : ~ ( ( short int ) ( 1 << 0 ) ) )  ; 
	RowRead_RR_C_LEFT = ( System_CALCULATOR ? ( 1 << 4 ) : ( 1 << 1 ) )  ; 
	RowRead_RR_R_UP = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 0 ) ) : ~ ( ( short int ) ( 1 << 0 ) ) )  ; 
	RowRead_RR_C_UP = ( System_CALCULATOR ? ( 1 << 5 ) : ( 1 << 0 ) )  ; 
	RowRead_RR_R_ENTER1 = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 9 ) ) : ~ ( ( short int ) ( 1 << 1 ) ) )  ; 
	RowRead_RR_C_ENTER1 = ( System_CALCULATOR ? ( 1 << 1 ) : ( 1 << 0 ) )  ; 
	RowRead_RR_R_ENTER2 = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 6 ) ) : ~ ( ( short int ) 0 ) )  ; 
	RowRead_RR_C_ENTER2 = ( System_CALCULATOR ? ( 1 << 6 ) : 0 )  ; 
	RowRead_RR_R_F4 = ( System_CALCULATOR ? ~ ( ( short int ) ( 1 << 9 ) ) : ~ ( ( short int ) ( 1 << 2 ) ) )  ; 
	RowRead_RR_C_F4 = ( System_CALCULATOR ? ( 1 << 4 ) : ( 1 << 7 ) )  ; 
	ProtArray [ 26 ] = CaptionedComponent = TCaptionedComponent0 ( ) ; 
	ProtArray [ 27 ] = IOStream = TIOStream0 ( ) ; 
	ProtArray [ 28 ] = Label = TLabel0 ( ) ; 
	ProtArray [ 29 ] = TextField = TTextField0 ( ) ; 
	ProtArray [ 30 ] = Menu = TMenu0 ( ) ; 
	ProtArray [ 31 ] = MenuItem = TMenuItem0 ( ) ; 
	ProtArray [ 32 ] = Vector = TVector0 ( ) ; 
	ProtArray [ 33 ] = Entry = TEntry0 ( ) ; 

atexit ( finalize ) ; 

#if MOKACLASS_GEM
	GEM_gem = GEM ; 
	
	MOKAMACRO_CALLMAIN ;
	
	GEM_gem -> setStarted_BOOL ( GEM_gem , TRUE ) ; 
	GEM_gem -> setStarted_BOOL ( GEM_gem , FALSE ) ; 
#else /* MOKACLASS_GEM */

	MOKAMACRO_CALLMAIN ( MOKAMACRO_MAINPARMS ) ;
	 
#endif /* MOKACLASS_GEM */
#ifdef RETURN_VALUE
	TRY
		while (GetArgType (top_estack) != END_TAG)
			top_estack = next_expression_index (top_estack);
		top_estack--;
	ONERR
	ENDTRY
	push_longint ( 0 ) ; 
#endif /* RETURN_VALUE */
	
} 
#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_Instantiation
/*
 * Used to determine dynamically if an object is an instance of a specified class or of a subclass of the specified class.
 */
BOOL instanceof ( TObject * object, TObject * prototype ) { 
	while ( object ) { 
		if ( object -> class_id == prototype -> class_id ) { 
			return TRUE ; 
		} 
		object = object -> super ; 
	} 
	return FALSE ; 
}
#endif
#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection || MOKAUSES_GarbageCollector

void ADT_GarbageCollector_init(ADT_GarbageCollector *this) {
	this->array = malloc(sizeof(void *) * ADT_GarbageCollector_INITIALCAPACITY);
	this->size  = 0;
	this->max   = ADT_GarbageCollector_INITIALCAPACITY;
}

/*void ADT_GarbageCollector_dispose(ADT_GarbageCollector *this) {
	free(this->array);
}*/

void * ADT_GarbageCollector_add(ADT_GarbageCollector *this, void *ptr) {
	if (this->size == this->max) {
		this->max += ADT_GarbageCollector_INCREMENT;
		this->array = realloc(this->array, (sizeof(void *) * this->max));
	}

	this->array[this->size++] = ptr;

	return ptr;
}

void * ADT_GarbageCollector_rem(ADT_GarbageCollector *this, void *ptr) {
	long int i = 0;

	while ((i < this->size) && (this->array[i] != ptr)) i++;

	if (i < this->size) {
		memmove(this->array + i, this->array + i + 1, (this->size - i - 1) * sizeof (void *));

		this->size--;
	}

	return ptr;
}

#if MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p
	void ADT_GarbageCollector_clearPtrs(ADT_GarbageCollector *this) {
		while (this->size) System_free_void_p(this->array[0]);

		free(this->array);
	}
#endif /* MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p */

void ADT_GarbageCollector_clearObjects(ADT_GarbageCollector *this) {
	#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection
		while (this->size) ((TObject *)this->array[0])->finalize_((TObject *)this->array[0]);
	#else
		while (this->size--) ((TObject *)this->array[this->size])->finalize_((TObject *)this->array[this->size]);
	#endif /* MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection */

	free(this->array);
}

#endif /* MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection || MOKAUSES_GarbageCollector */
/*
 * This function is called when the program exits.
 * It frees the ressources used by the prototypes of the objects.
 */
void CALLBACK finalize ( void ) { 

	if ( ProtArray ) 
		free ( ProtArray ) ; 
	free ( GuiASM ) ; 
	DisASM_finally_ ( ) ; 
	free ( Format ) ; 
	free ( FormatRR0 ) ; 
	FormatRR0_finally_ ( ) ; 
	free ( FormatRR1 ) ; 
	FormatRR1_finally_ ( ) ; 
	free ( FormatRS0 ) ; 
	FormatRS0_finally_ ( ) ; 
	free ( FormatRS1 ) ; 
	FormatRS1_finally_ ( ) ; 
	free ( FormatRS2 ) ; 
	FormatRS2_finally_ ( ) ; 
	free ( FormatRX0 ) ; 
	FormatRX0_finally_ ( ) ; 
	free ( FormatSI0 ) ; 
	FormatSI0_finally_ ( ) ; 
	free ( FormatSS0 ) ; 
	FormatSS0_finally_ ( ) ; 
	free ( FormatSS1 ) ; 
	FormatSS1_finally_ ( ) ; 
	free ( FrameAbout ) ; 
	free ( FrameASM ) ; 
	free ( String ) ; 
	free ( Frame ) ; 
	GEM -> finalize_ ( GEM ) ; 
	free ( Button ) ; 
	free ( TextArea ) ; 
	free ( File ) ; 
	free ( OptionPane ) ; 
	OptionPane_finally_ ( ) ; 
	free ( Object ) ; 
	free ( Container ) ; 
	free ( MenuBar ) ; 
	free ( Component ) ; 
	free ( EventListener ) ; 
	free ( Interrupt ) ; 
	Interrupt_finally_ ( ) ; 
	free ( CaptionedComponent ) ; 
	free ( IOStream ) ; 
	free ( Label ) ; 
	free ( TextField ) ; 
	free ( Menu ) ; 
	free ( MenuItem ) ; 
	free ( Vector ) ; 
	free ( Entry ) ; 

#if MOKASTATICATTRIBUTE_GEM_vScrRect
	free ( GEM_vScrRect ) ; 
#else  /* MOKASTATICATTRIBUTE_GEM_vScrRect */
#if MOKASTATICATTRIBUTE_Graphics_uScrRect
	free( Graphics_uScrRect ) ; 
#endif
#endif /* MOKASTATICATTRIBUTE_GEM_vScrRect */
#if ! MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_SaveScreen
	LCD_restore ( systemScrSaveBuffer ) ; 
	free ( systemScrSaveBuffer ) ; 
#endif /* MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_No_SaveScreen */
#if MOKASTATICMETHOD_System_getMainArgs_
	short int i;
	for (i = 0; i < System_argCount; i++) {
		ArgsArray[i]->finalize_(ArgsArray[i]);
	}

	if ( ArgsArray )
		free ( ArgsArray ) ;
#endif /* MOKASTATICMETHOD_System_getMainArgs_ */

#if MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection
	ADT_GarbageCollector_clearObjects(& Objects_GarbageCollector_instance);
	
	#if MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p
		ADT_GarbageCollector_clearPtrs(& Ptrs_GarbageCollector_instance);
	#endif /* MOKASTATICMETHOD_System_malloc_long_int || MOKASTATICMETHOD_System_free_void_p */
#endif /* MOKACLASS_MAINCLASS_IMPLEMENTSINTERFACE_Use_GarbageCollection */
} 
