//Defining class Component
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Component is the base class for user-interface objects.
 *
 * @author  Frdric Brown
 * @version 1.01, 2003-11-02
 * @since   MDK1.0a
 */
TComponent * TComponent0 ( ) {
	TComponent * this = malloc ( sizeof ( TComponent ) ) ; 
	this -> class = ( char * ) "Component" ; 
	this -> class_id = 22 ; 
	this -> super = ( void * ) EventListener ; 
	//Methods from class Object
	this -> toString_ = ( void * ) TObject_toString_ ; 
	//Methods from class EventListener
	//Methods from class Component
	this -> finalize_ = ( void * ) TComponent_finalize_ ; 
	this -> eventTriggered_Object = ( void * ) TComponent_eventTriggered_Object ; 
	this -> onPaint_ = ( void * ) TComponent_onPaint_ ; 
	this -> paint_ = ( void * ) TComponent_paint_ ; 
	this -> setBounds_short_int_short_int_short_int_short_int = ( void * ) TComponent_setBounds_short_int_short_int_short_int_short_int ; 
	this -> setY_short_int = ( void * ) TComponent_setY_short_int ; 
	this -> move_short_int_short_int = ( void * ) TComponent_move_short_int_short_int ; 
	this -> setVisible_BOOL = ( void * ) TComponent_setVisible_BOOL ; 
	this -> use_ = ( void * ) TComponent_use_ ; 
	this -> isCursorOver_short_int_short_int = ( void * ) TComponent_isCursorOver_short_int_short_int ; 
	this -> center_ = ( void * ) TComponent_center_ ; 
	this -> toFront_Component = ( void * ) TComponent_toFront_Component ; 
	this -> clean_ = ( void * ) TComponent_clean_ ; 
	this -> checkMnemonic_char = ( void * ) TComponent_checkMnemonic_char ; 
	return this ; 
} 
/**
	 * Constructs a newly allocated Component.
	 */
TComponent * TComponent_ ( ) {
	TComponent * this = TComponent0 ( ) ; 
	Component__ ( this ) ; 
	return this ; 
} 
void Component__ ( struct Component * this ) { 
	this -> img = NULL ; 
	this -> repaint = TRUE ; 
	this -> parent = NULL ; 
	this -> mnemonic = '\0' ; 
	this -> height = 25 ; 
	this -> width = 25 ; 
	this -> x = 0 ; 
	this -> y = 0 ; 
	this -> visible = TRUE ; 
	this -> enabled = TRUE ; 
	this -> listen = this ; 
	
} 
/**
     * Free the memory and system ressources used by this Component.
     */
void TComponent_finalize_ ( struct Component * this ) { 
if (this->img) {
			free(this->img);
		}
		free(this);
	
} 
/**
     * Invoked when an event is triggered.
     *
     * @param sender The object who triggered the event.
     */
void TComponent_eventTriggered_Object ( struct Component * this , struct Object * sender ) { 
	
} 
/**
	 * Called when a PaintEvent occurs to paint the image of the component and
	 * stores it in its virtual memory.  A PaintEvent occurs when the aspect of
	 * the component is altered.
	 */
void TComponent_onPaint_ ( struct Component * this ) { 
this->repaint = FALSE;
		SCR_RECT rect = (SCR_RECT){{0, 0, this->width - 1, this->height - 1}};

		PortSet(GEM_gem->port, 239, 127);
		ScrRectFill (&rect, &rect, A_REVERSE);
		
		this->paint_(this);
		
		if (this->img) {
			free (this->img);
		}
		
		this->img = malloc(BitmapSize(&rect));
		
		BitmapGet (&rect, this->img);
		PortSet ((void *) 0x4C00, 239, 127);
	
} 
/**
	 * This method is called to draw the component.  Subclasses of Component
	 * are encouraged to overrides the <code>paint</code> method.
	 */
void TComponent_paint_ ( struct Component * this ) { 
	
} 
/*
	 * Sets the bounds (position and size) of the component.
	 *
	 * @param x the horizontal coordinate.
	 * @param y the vertical coordinate.
	 * @param width the width of the component.
	 * @param height the height of the component.
	 */
void TComponent_setBounds_short_int_short_int_short_int_short_int ( struct Component * this , short int x , short int y , short int width , short int height ) { 
	this -> x = x ; 
	this -> y = y ; 
	this -> width = width ; 
	this -> height = height ; 
	this -> repaint = TRUE ; 
	if ( this -> visible && this -> parent ) { 
		this -> parent -> onRefresh_ ( this -> parent ) ; 
		
	} 
	
} 
/*
	 * Sets the vertical coordinate of the component.
	 *
	 * @param y the vertical coordinate.
	 */
void TComponent_setY_short_int ( struct Component * this , short int y ) { 
	this -> y = y ; 
	if ( this -> visible && this -> parent ) { 
		this -> parent -> onRefresh_ ( this -> parent ) ; 
		
	} 
	
} 
/*
	 * Sets the position of the component.
	 *
	 * @param x the horizontal coordinate.
	 * @param y the vertical coordinate.
	 */
void TComponent_move_short_int_short_int ( struct Component * this , short int x , short int y ) { 
	this -> x = x ; 
	this -> y = y ; 
	if ( this -> visible && this -> parent ) { 
		this -> parent -> onRefresh_ ( this -> parent ) ; 
		
	} 
	
} 
/**
	 * Sets the visibility of the component.
	 * 
	 * @param b true if the component should be visible, false otherwize.
	 */
void TComponent_setVisible_BOOL ( struct Component * this , BOOL b ) { 
	this -> visible = b ; 
	if ( this -> parent ) { 
		this -> parent -> onRefresh_ ( this -> parent ) ; 
		
	} 
	
} 
/**
	 * This method is invoked by the GEM when the user interacts
	 * with the component.
	 */
void TComponent_use_ ( struct Component * this ) { 
	if ( this -> enabled && this -> listen ) { 
		this -> listen -> eventTriggered_Object ( this -> listen , this ) ; 
		
	} 
	
} 
/**
	 * Indicates if the cursor is over this component.
	 *
	 * @param x the reference's horizontal coordinate
	 * @param y	the reference's vertical coordinate
	 * @return true if the cursor is over the component, false otherwize.
	 */
BOOL TComponent_isCursorOver_short_int_short_int ( struct Component * this , short int x , short int y ) { 
	return GEM_gem -> cursorX >= this -> x + x && GEM_gem -> cursorY >= this -> y + y && GEM_gem -> cursorX < this -> x + x + this -> width && GEM_gem -> cursorY < this -> y + y + this -> height ; 
	
} 
/*
	 * Centers the component in its parent.
	 */
void TComponent_center_ ( struct Component * this ) { 
	this -> x = ( this -> parent -> width - this -> width ) / 2 ; 
	this -> y = ( this -> parent -> height - this -> height ) / 2 ; 
	if ( this -> visible && this -> parent ) { 
		this -> parent -> onRefresh_ ( this -> parent ) ; 
		
	} 
	
} 
/**
	 * Called to brings a component to the front
	 * of a container.
	 * 
	 * @param c the component to bring to front.
	 * @return this component if it is the component to bring to front, null otherwize.
	 */
TComponent *  TComponent_toFront_Component ( struct Component * this , struct Component * c ) { 
	if ( c == this ) { 
		return c ; 
		
	} 
	return NULL ; 
	
} 
/**
	 * Disposes of all ressources used by this component.
	 */
void TComponent_clean_ ( struct Component * this ) { 
	this -> finalize_ ( this ) ; 
	
} 
/**
	 * Checks if the mnemonic of this component correspond
	 * to the current mnemonic. If it is the case, an event
	 * is triggered.
	 */
void TComponent_checkMnemonic_char ( struct Component * this , char m ) { 
	if ( this -> listen && this -> mnemonic == m ) { 
		//this.listen.eventTriggered(this); 
		this -> use_ ( this ) ; 
		
	} 
	
} 
